package com.dskj.znzn.importData.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dskj.znzn.importData.common.exception.BusinessException;
import com.dskj.znzn.importData.web.entity.BaseTemplate;
import com.dskj.znzn.importData.web.entity.BaseTemplateField;
import com.dskj.znzn.importData.web.entity.ProcessGroup;
import com.dskj.znzn.importData.web.indata.AddBaseTemplateInData;
import com.dskj.znzn.importData.web.indata.EditBaseTemplateInData;
import com.dskj.znzn.importData.web.indata.TemplateFieldsInData;
import com.dskj.znzn.importData.web.mapper.BaseTemplateMapper;
import com.dskj.znzn.importData.web.outdata.GetBaseTemplateOutData;
import com.dskj.znzn.importData.web.service.IBaseTemplateFieldService;
import com.dskj.znzn.importData.web.service.IBaseTemplateService;
import com.dskj.znzn.importData.web.service.IProcessGroupService;
import lombok.RequiredArgsConstructor;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author 魏志亮
 * @since 2024-10-29
 */
@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired}))
public class BaseTemplateServiceImpl extends ServiceImpl<BaseTemplateMapper, BaseTemplate> implements IBaseTemplateService {

    private final IBaseTemplateFieldService baseTemplateFieldService;
    private final IProcessGroupService processGroupService;

    @Override
    public List<BaseTemplate> getAllEnabled(String name) {

        LambdaQueryWrapper<BaseTemplate> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.ne(BaseTemplate::getStatus, 0);
        if (StringUtils.isNotBlank(name)) {
            queryWrapper.like(BaseTemplate::getName, name);
        }
        return list(queryWrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addTemplate(AddBaseTemplateInData inData) {
        LambdaQueryWrapper<BaseTemplate> baseWrapper = Wrappers.lambdaQuery();
        baseWrapper.eq(BaseTemplate::getName, inData.getName()).ne(BaseTemplate::getStatus, 0);
        if (count(baseWrapper) > 0) {
            throw new BusinessException("目标名称已存在");
        }

        BaseTemplate baseTemplate = new BaseTemplate();
        baseTemplate.setName(inData.getName());
        save(baseTemplate);

        List<BaseTemplateField> fieldsList = getBaseTemplateFields(inData, baseTemplate);
        baseTemplateFieldService.saveBatch(fieldsList, 100);
    }

    private static @NotNull List<BaseTemplateField> getBaseTemplateFields(AddBaseTemplateInData inData, BaseTemplate baseTemplate) {
        Integer id = baseTemplate.getId();
        List<BaseTemplateField> fieldsList = new ArrayList<>();
        for (TemplateFieldsInData fields : inData.getTemplateFieldsInDataList()) {
            BaseTemplateField baseTemplateField = new BaseTemplateField();
            baseTemplateField.setTemplateId(id);
            baseTemplateField.setStandardizedName(fields.getStandardizedName());
            baseTemplateField.setIsIndex(fields.getIsIndex());
            baseTemplateField.setIsCutCol(fields.getIsCutCol());
            fieldsList.add(baseTemplateField);
        }
        return fieldsList;
    }

    @Override
    public void deleteTemplate(Integer id) {
        BaseTemplate baseTemplate = getById(id);
        if (null == baseTemplate || baseTemplate.getStatus() == 0) {
            throw new BusinessException("查询模板不存在或已删除");
        }
        if (baseTemplate.getStatus() == 2) {
            throw new BusinessException("默认模板不允许删除");
        }

        baseTemplate = new BaseTemplate();
        baseTemplate.setId(id);
        baseTemplate.setStatus(0);
        updateById(baseTemplate);
    }

    @Override
    public void queryUsed(Integer id) {
        BaseTemplate baseTemplate = getById(id);
        if (null == baseTemplate || baseTemplate.getStatus() == 0) {
            throw new BusinessException("查询模板不存在或已删除");
        }

        List<ProcessGroup> usedGroupList = processGroupService.getByTemplateId(id);
        if (!usedGroupList.isEmpty()) {
            String names = usedGroupList.stream().map(ProcessGroup::getName).collect(Collectors.joining(","));
            throw new BusinessException("模板已被:[" + names + "]使用");
        }
    }

    @Override
    public GetBaseTemplateOutData getBaseDataById(Integer id) {
        BaseTemplate baseTemplate = getById(id);
        if (null == baseTemplate || baseTemplate.getStatus() == 0) {
            throw new BusinessException("查询模板不存在或已删除");
        }
        List<BaseTemplateField> fieldList = baseTemplateFieldService.getByTemplateId(id);
        List<ProcessGroup> groupList = processGroupService.getByTemplateId(id);

        GetBaseTemplateOutData data = new GetBaseTemplateOutData();
        data.setBaseTemplate(baseTemplate);
        data.setFieldList(fieldList);
        data.setUseTemplateGroupList(groupList);

        return data;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void editTemplate(EditBaseTemplateInData inData) {
        Integer id = inData.getId();
        BaseTemplate baseTemplate = getById(id);
        if (null == baseTemplate || baseTemplate.getStatus() == 0) {
            throw new BusinessException("查询模板不存在或已删除");
        }

        if (baseTemplate.getStatus() == 2) {
            throw new BusinessException("默认模板不允许编辑");
        }

        List<ProcessGroup> usedGroupList = processGroupService.getByTemplateId(id);
        if (!usedGroupList.isEmpty()) {
            String names = usedGroupList.stream().map(ProcessGroup::getName).collect(Collectors.joining(","));
            throw new BusinessException("模板已被:[" + names + "]使用");
        }
        if (!StringUtils.equals(inData.getName(), baseTemplate.getName())) {
            BaseTemplate update = new BaseTemplate();
            update.setId(id);
            update.setName(inData.getName());
            updateById(update);
        }

        baseTemplateFieldService.deleteAndSaveByTemplateId(id, inData.getTemplateFieldsInDataList());
    }
}
